package Handler::Javascript::Parser;

use strict;

use URI;
use HTML::Parser;
use LWP::UserAgent;
use HTTP::Request;

use Handler::Javascript::Interpreter;

use Class::MethodMaker
    [
        scalar => [ qw/ interpreter environment parser document_root filename interpreting / ],
        new    => [ qw/ -init new / ],
    ];

sub init
{
    my ($self, $environment) = @_;
    
    $self->{output} = '';
    $self->{input} = undef;
    
    $self->environment($environment);
    $self->document_root($environment->{DOCUMENT_ROOT});
    $self->filename($environment->{SCRIPT_FILENAME});

    $self->{input} = $self->_read_file_local($self->filename);
    return unless $self->{input};

    $self->interpreter(Handler::Javascript::Interpreter->new());
}

sub parse
{
    my ($self) = @_;

    return unless $self->{input};
        
    my $parser = HTML::Parser->new(
        api_version => 3,
        text_h      => [ sub { $self->_text_handler(@_); },  'text' ],
        start_h     => [ sub { $self->_start_handler(@_); }, 'tagname,attr'],
        end_h       => [ sub { $self->_end_handler(@_); },   'tagname,attr' ]
    );

    $parser->empty_element_tags(1);
    $parser->unbroken_text(1);
    
    $parser->parse($self->{input});
}

sub input
{
    my ($self) = @_;
    
    return $self->{input};
}

sub output
{
    my ($self) = @_;
    
    return $self->{output};
}

sub _read_file
{
    my ($self, $uri) = @_;
    
    # supported types: file, http
    my $u = URI->new($uri);
    
    if ($u->scheme eq 'http') {
        return $self->_read_file_http($uri);
    } elsif ($u->scheme eq 'file' || !defined($u->scheme)) {
        my $filename = $u->path;
        $filename =~ s/..\///g;
        $filename =~ s/$\///g;

        my $escaped_filename = $self->document_root() . "/" . $filename;
        return $self->_read_file_local($escaped_filename);
    }
}

sub _read_file_local
{
    my ($self, $filename) = @_;
    
    open(IN, "<$filename") || return;
    
    my $content = '';
    while (<IN>) {
        $content .= $_;
    }
    close(IN);
    
    return $content;
}

sub _read_file_http
{
    my ($self, $uri) = @_;

    my $user_agent = LWP::UserAgent->new();
    $user_agent->timeout(1800);
    $user_agent->env_proxy;

    my $request = HTTP::Request->new(GET => $uri);

    my $res = $user_agent->request($request);
    
    return $res->content
        if ($res->is_success);
}

sub _text_handler
{
    my ($self, $text) = @_;

    if ($self->interpreting) {
        $self->{torun} .= $text;
    } else {
        $self->{output} .= $text;
    }
}

sub _start_handler
{
    my ($self, $tag, $attr) = @_;
    
    if ($tag eq 'script' && $attr->{type} eq 'text/javascript' && $attr->{runat} eq 'server') {
        $self->{torun} = '';
        if ($attr->{src}) {
            $self->{torun} = $self->_read_file($attr->{src});
        }
        $self->interpreting(1);
    } else {
        $self->{output} .= "<" . $tag . ">";
    }
}

sub _end_handler
{
    my ($self, $tag) = @_;

    if ($tag eq 'script' && $self->interpreting) {
        $self->interpreting(0);
        $self->interpreter->interpret($self->{torun});
        $self->{output} .= $self->interpreter->output();
        $self->{torun} = '';
    } else {
        $self->{output} .= "</" . $tag . ">";
    }
}

1;